/****************************************************************************
 * PCM Playback example
 * Nebiun 2020
 ***************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <gccore.h>
#include <wiiuse/wpad.h>
#include <asndlib.h>

// include generated header
#include "gear_raw.h"
#include "hit_raw.h"

static void *xfb = NULL;
static GXRModeObj *rmode = NULL;

//---------------------------------------------------------------------------------
int main(int argc, char **argv) 
{
//---------------------------------------------------------------------------------
	void *sounds[2];
	size_t sound_size[2];
	int i;
	
	// Initialise the video system
	VIDEO_Init();

	// Initialise the attached controllers
	WPAD_Init();

	// Initialise the audio subsystem
	ASND_Init();

	// Obtain the preferred video mode from the system
	// This will correspond to the settings in the Wii menu
	rmode = VIDEO_GetPreferredMode(NULL);

	// Allocate memory for the display in the uncached region
	xfb = MEM_K0_TO_K1(SYS_AllocateFramebuffer(rmode));

	// Initialise the console, required for printf
	console_init(xfb,20,20,rmode->fbWidth,rmode->xfbHeight,rmode->fbWidth*VI_DISPLAY_PIX_SZ);

	// Try with aligned buffer multiplied of 32 bytes, as manual said, but no luck.
	sound_size[0] = ((gear_raw_size + 31)/32) * 32;
	sound_size[1] = ((hit_raw_size + 31)/32) * 32;
	for(i=0; i< 2; i++) {
		sounds[i] = memalign(32,sound_size[i]);
		memset(sounds[i],0,sound_size[i]);
	}
	memmove(sounds[0], gear_raw, gear_raw_size);
	memmove(sounds[1], hit_raw, hit_raw_size);
	//-------
	
	// Set up the video registers with the chosen mode
	VIDEO_Configure(rmode);

	// Tell the video hardware where our display memory is
	VIDEO_SetNextFramebuffer(xfb);

	// Make the display visible
	VIDEO_SetBlack(FALSE);

	// Flush the video register changes to the hardware
	VIDEO_Flush();

	// Wait for Video setup to complete
	VIDEO_WaitVSync();
	if(rmode->viTVMode&VI_NON_INTERLACE) VIDEO_WaitVSync();

	// The console understands VT terminal escape codes
	// This positions the cursor on row 2, column 0
	// we can use variables for this with format codes too
	// e.g. printf ("\x1b[%d;%dH", row, column );
	printf("\x1b[2;0H");

	printf("Playing sample RAW file...Press HOME to exit.\n");

	if(ASND_SetInfiniteVoice(0, VOICE_MONO_8BIT, 11025, 0, (void *)sounds[0], sound_size[0], MAX_VOLUME, MAX_VOLUME) != SND_OK)
		printf("ASND_SetInfiniteVoice error\n");
	while(1) {

		// Call WPAD_ScanPads each loop, this reads the latest controller states
		WPAD_ScanPads();
		
		// WPAD_ButtonsDown tells us which buttons were pressed in this loop
		// this is a "one shot" state which will not fire again until the button has been released
		u32 pressed = WPAD_ButtonsDown(0);

		if (pressed & WPAD_BUTTON_A) {
			printf("Bing %d\n", hit_raw_size);
			if(ASND_SetVoice(1, VOICE_MONO_8BIT, 11025, 0, (void *)sounds[1], sound_size[1],  MAX_VOLUME, MAX_VOLUME, NULL) != SND_OK)
				printf("ASND_SetVoice error\n");	
		}
		// We return to the launcher application via exit
		else if ( pressed & WPAD_BUTTON_HOME ) break;

		// Wait for the next frame
		VIDEO_WaitVSync();
	}
	ASND_StopVoice(0);
	ASND_StopVoice(1);
	return 0;
}
